import apache_beam as beam

class Step(beam.DoFn):
  def process(self, element):
    raise NotImplementedError("Subclasses must implement this method.")

  def name(self):
    raise NotImplementedError("Subclasses must implement this method.")





class Dataset:
  def __init__(self):
    raise NotImplementedError("Subclasses must implement this method.")
  
  def __getitem__(self, index):
    raise NotImplementedError("Subclasses must implement this method.")
    
  def __len__(self):
    raise NotImplementedError("Subclasses must implement this method.")



from fastavro import parse_schema
from atelierflow.experiments import Experiments

class ExperimentBuilder:
    def __init__(self):
        self.experiments = Experiments()

    def add_model(self, model):
        self.experiments.add_model(model)
        return self

    def add_metric(self, metric):
        self.experiments.add_metric(metric)
        return self

    def add_train_dataset(self, train_dataset):
        self.experiments.add_train(train_dataset)
        return self

    def add_test_dataset(self, test_dataset):
        self.experiments.add_test(test_dataset)
        return self

    def add_step(self, step):
        self.experiments.add_step(step)
        return self

    def set_avro_schema(self, avro_schema):
        self.experiments.avro_schema = parse_schema(avro_schema)
        return self
    
    def set_cross_validation(self, boolean):
        self.experiments.cross_validation = boolean
        return self

    def build(self):
        return self.experiments


from .model import BaseModel
from .experiments import Experiments
from .experimentsBuilder import ExperimentBuilder
from .metrics.metric import BaseMetric
from .datasets.dataset import Dataset
from .datasets.acoustic_dataset import AcousticDataset
from .steps.step import Step

__all__ = [
  "BaseModel",
  "Experiments",
  "BaseMetric",
  "Dataset",
  "ExperimentBuilder",
  "AcousticDataset",
  "Step"
]



import apache_beam as beam
from apache_beam.options.pipeline_options import PipelineOptions

def printRecord(element):
    for key, value in element.items():
        print(f"{key}: {value}")
    print("-" * 40)


class CustomPipeline:
    def __init__(self, file_path) -> None:
        self.file_path = file_path

    def run(self):
        options = PipelineOptions()
        with beam.Pipeline(options=options) as p:
            _ = (
                p
                | "Read" >> beam.io.ReadFromAvro(self.file_path)
                | "Print Records" >> beam.Map(printRecord)
            )

